from pydantic import BaseModel, field_validator
from typing import Optional, Literal


class Adr(BaseModel):
    satzart: Literal["ADR"] = "ADR"
    referenz: Optional[str] = None
    tladenr: Optional[str] = None
    aufnr: Optional[int] = None
    kunart: Optional[int] = None
    kundennr: Optional[int] = None
    iln: Optional[str] = None
    name1: Optional[str] = None
    name2: Optional[str] = None
    strasse: Optional[str] = None
    lkz: Optional[str] = None
    plz: Optional[str] = None
    ort: Optional[str] = None
    ortt: Optional[str] = None
    pfach: Optional[str] = None
    pfachplz: Optional[str] = None
    tel: Optional[str] = None
    fax: Optional[str] = None
    mc: Optional[str] = None
    partner: Optional[str] = None
    kontotab: Optional[int] = None
    kstelle: Optional[int] = None
    bundesland: Optional[str] = None
    wadrid: Optional[str] = None
    wupddatum: Optional[str] = None
    ktraeger: Optional[int] = None
    ustnr: Optional[str] = None
    uscheidnr: Optional[int] = None
    zollnr: Optional[str] = None
    gruppe: Optional[str] = None
    abholff: Optional[int] = None
    emailadr: Optional[str] = None
    statistik1: Optional[str] = None
    statistik2: Optional[str] = None
    statistik3: Optional[str] = None
    statistik4: Optional[str] = None
    statistik5: Optional[str] = None
    name3: Optional[str] = None
    orgkundennr: Optional[int] = None
    kuntemp: Optional[str] = None
    ortgeox: Optional[int] = None
    ortgeoy: Optional[int] = None
    ortgeoxk: Optional[int] = None
    ortgeoyk: Optional[int] = None
    ortelfdnr: Optional[int] = None
    kundennra: Optional[str] = None
    ktodeb: Optional[int] = None
    ktokre: Optional[int] = None
    telauto: Optional[str] = None
    uc: Optional[str] = None
    ustid: Optional[str] = None
    steuernr: Optional[str] = None
    zahlbed: Optional[str] = None
    frankatur: Optional[str] = None
    verkart: Optional[str] = None
    sprache: Optional[int] = None
    reperiode: Optional[str] = None
    rewaehr: Optional[str] = None
    gsperiode: Optional[str] = None
    gswaehr: Optional[str] = None
    konzernnr: Optional[int] = None
    abwrekunnr: Optional[int] = None
    abwgskunnr: Optional[int] = None
    homepage: Optional[str] = None
    info: Optional[str] = None
    cmrzubis: Optional[str] = None
    cmrzubetr: Optional[float] = None
    cmrzuwaeh: Optional[str] = None
    kvobis: Optional[str] = None
    kvobetrag: Optional[float] = None
    kvowaehr: Optional[str] = None
    cmrdatum: Optional[str] = None
    dmrbetrag: Optional[float] = None
    cmrwaehr: Optional[str] = None
    extadrnr: Optional[str] = None
    ppbetragre: Optional[float] = None
    ppbetraggs: Optional[float] = None
    kabodat: Optional[str] = None
    adrarteig: Optional[str] = None
    adrreflfd: Optional[int] = None
    ktodeba: Optional[str] = None
    ktokrea: Optional[str] = None
    borderonr: Optional[str] = None
    milogdat: Optional[str] = None
    uiccountry: Optional[int] = None
    uicnr: Optional[int] = None
    uirrnr: Optional[int] = None
    opsaldo: Optional[float] = None
    kredlimit: Optional[float] = None
    guthsaldo: Optional[float] = None
    guthlimit: Optional[float] = None
    limitkz: Optional[int] = None
    sperre: Optional[bool] = None
    sperrdatum: Optional[str] = None
    sperrgrund: Optional[str] = None
    bonitaet: Optional[str] = None
    bonitaetsgrenze: Optional[str] = None
    locode: Optional[str] = None

    @field_validator("sperre")
    @classmethod
    def validate_boolean_str(cls, value):
        if value in {True, "J", "j"}:
            return "J"
        elif value in {False, "N", "n"}:
            return "N"
        elif value is None:
            return None
        raise ValueError("sperre must be 'J', 'N', True, False or None")



from pydantic import BaseModel, Field
from typing import Literal, Optional


class DmsSw(BaseModel):
    satzart: Literal["DMSSW"] = "DMSSW"
    referenz: str
    dmsdoknr: int
    dmsswnr: int
    schluessel: str
    wert: str



from pydantic import BaseModel
from typing import Literal, Optional


class DmsDok(BaseModel):
    satzart: Literal["DMSDOK"] = "DMSDOK"
    referenz: str
    dmsdoknr: int
    archiv: str
    ordner: str
    doktyp: str
    quellpfad: str
    aendstatus: Optional[str] = None



from typing import List
from datetime import datetime, time
from pydantic import BaseModel

from .types import *


class LisOut(BaseModel):
    def __init__(self, *args, **kwargs):
        raise NotImplementedError()



from datetime import datetime
from typing import Any, Optional, Union
import xml.etree.ElementTree as ET
from pydantic import BaseModel
import pytz


class XmlAttribute:
    """A marker type for XML attributes."""

    def __init__(self, value: Any):
        self.value = value


def to_xml_element(
    tag: str, value: Union[str, int, float, bool, datetime, list, BaseModel, None]
) -> Optional[ET.Element]:
    """Convert a Pydantic model or a primitive type to an XML element."""
    if isinstance(value, BaseModel):
        element = ET.Element(tag)
        for field_name, field_value in value:
            if isinstance(field_value, XmlAttribute):
                element.set(field_name, str(field_value.value))
            else:
                child_element = to_xml_element(field_name, field_value)
                if child_element is not None:
                    element.append(child_element)
    elif isinstance(value, list):
        element = ET.Element(tag)
        for item in value:
            item_tag = item.__class__.__name__
            child_element = to_xml_element(item_tag, item)
            if child_element is not None:
                element.append(child_element)
    elif isinstance(value, datetime):
        if value.tzinfo is None:
            # Assume CET/CEST
            cet = pytz.timezone("Europe/Berlin")
            value = cet.localize(value)

        # Format the datetime explicitly
        iso_format = value.strftime("%Y-%m-%dT%H:%M:%S%z")
        iso_format = (
            iso_format[:-2] + ":" + iso_format[-2:]
        )  # Add colon in the timezone part

        element = ET.Element(tag)
        element.text = iso_format
    elif value is None:
        return None
    else:
        element = ET.Element(tag)
        element.text = str(value)

    return element



from pydantic import BaseModel, Field
from typing import Literal, Optional


class DmsRef(BaseModel):
    satzart: Literal["DMSREF"] = "DMSREF"
    referenz: str
    dmsdoknr: int
    dmsrefnr: int
    aufnr: int
    kundennr: Optional[str] = None
    liefnr: Optional[str] = None
    refnr: Optional[str] = None
    kommnr: Optional[str] = None



import os
import pysftp

from typing import Optional
from pydantic import BaseModel
from pydantic.dataclasses import dataclass


class SftpConfig(BaseModel):
    host: str
    port: int = 22
    username: Optional[str] = None
    password: Optional[str] = None
    private_key: Optional[str] = None
    private_key_pass: Optional[str] = None
    no_host_key: bool = False
    cnopts: Optional[pysftp.CnOpts] = None

    class Config:
        arbitrary_types_allowed = True


@dataclass
class SftpBase:
    config: SftpConfig

    def __post_init__(self):
        if self.config.no_host_key:
            self.config.cnopts = pysftp.CnOpts()
            self.config.cnopts.hostkeys = None
        conn = pysftp.Connection(
            host=self.config.host,
            port=self.config.port,
            username=self.config.username,
            password=self.config.password,
            private_key=self.config.private_key,
            private_key_pass=self.config.private_key_pass,
            cnopts=self.config.cnopts,
        )
        conn.close()

    def import_file(self, source_filepath: str, dest_path: str) -> None:
        """
        Uploads a file from the source path to the destination path on the SFTP server.

        Args:
            credentials: The credentials required to establish an SFTP connection.
            source_filepath: The local path of the file to be uploaded.
            dest_path: The destination path on the SFTP server where the file will be uploaded.
        """
        with pysftp.Connection(
            host=self.config.host,
            port=self.config.port,
            username=self.config.username,
            password=self.config.password,
            private_key=self.config.private_key,
            private_key_pass=self.config.private_key_pass,
            cnopts=self.config.cnopts,
        ) as sftp:
            filename = os.path.basename(source_filepath)
            dest_filepath = f"{dest_path}/{filename}"
            sftp.put(source_filepath, dest_filepath)

    def export_file(self, remote_filepath: str, local_filepath: str) -> None:
        """
        Export a file from a remote SFTP server using the provided credentials.

        Args:
            remote_filepath (str): The file path on the remote SFTP server.
            local_filepath (str): The local file path where the downloaded file will be saved.
        """
        with pysftp.Connection(
            host=self.config.host,
            port=self.config.port,
            username=self.config.username,
            password=self.config.password,
            private_key=self.config.private_key,
            private_key_pass=self.config.private_key_pass,
            cnopts=self.config.cnopts,
        ) as sftp:
            sftp.get(remote_filepath, local_filepath)

    def get_all_files(self, remote_folder: str):
        """Export all the files from a specific remove SFTP direction using the provided credentials

        Args:
            remote_folder (str): The directory from which the files should be taken
            local_folder (str): The local folder path where the downloaded files will be saved.
        """
        remote_files = []

        with pysftp.Connection(
            host=self.config.host,
            port=self.config.port,
            username=self.config.username,
            password=self.config.password,
            private_key=self.config.private_key,
            private_key_pass=self.config.private_key_pass,
            cnopts=self.config.cnopts,
        ) as sftp:
            with sftp.cd(remote_folder):
                files = sftp.listdir()
                for file_name in files:
                    remote_filepath = f"{remote_folder}/{file_name}"
                    remote_files.append(remote_filepath)

        return remote_files

    def delete_file(self, remote_filepath: str):
        """Remove a file from a remote SFTP sever

        Args:
            remote_filepath (str): the file path ont he remove SFTP sever
        """
        with pysftp.Connection(
            host=self.config.host,
            port=self.config.port,
            username=self.config.username,
            password=self.config.password,
            private_key=self.config.private_key,
            private_key_pass=self.config.private_key_pass,
            cnopts=self.config.cnopts,
        ) as sftp:
            if sftp.exists(remote_filepath):
                sftp.remove(remote_filepath)



from pydantic import BaseModel, field_validator
from typing import Optional, Literal, Union
from datetime import datetime


class Ladeli(BaseModel):
    satzart: Literal["LADELI"] = "LADELI"
    referenz: str
    tladenr: str
    lkwpolkz: Optional[str] = None
    anhpolkz: Optional[str] = None
    fahid: Optional[int] = None
    fahname: Optional[str] = None
    ffiln: Optional[str] = None
    ffempid: Optional[str] = None
    ffname: Optional[str] = None
    abruf: Optional[str] = None
    tournr: Optional[str] = None
    tsmittel: Optional[str] = None
    exportiert: Optional[Union[str, bool]] = None
    wtourid: Optional[int] = None
    borderonr: Optional[str] = None
    tlade_datum: Optional[datetime] = None
    wechselbruecke1: Optional[str] = None
    wechselbruecke2: Optional[str] = None
    telefon: Optional[str] = None
    kmlast: Optional[float] = None
    kmmaut: Optional[float] = None
    dispo_user: Optional[str] = None
    ffpausch: Optional[float] = None
    lkwgrpid: Optional[str] = None
    tourzeit: Optional[str] = None
    entbisdat: Optional[datetime] = None
    entbiszeit: Optional[str] = None
    verkart: Optional[str] = None
    lposnr: Optional[str] = None
    lpospausch: Optional[float] = None
    tourinfo1: Optional[str] = None
    tourinfo2: Optional[str] = None
    tourinfo3: Optional[str] = None
    tourinfo4: Optional[str] = None
    tourinfo5: Optional[str] = None
    tourinfo6: Optional[str] = None
    tourinfo7: Optional[str] = None
    tourinfo8: Optional[str] = None
    tourinfo9: Optional[str] = None
    tourinfo10: Optional[str] = None
    dispous: Optional[str] = None
    fahvorname: Optional[str] = None
    bfahid: Optional[int] = None
    bfahname: Optional[str] = None
    bfahvorname: Optional[str] = None
    ffpwaehr: Optional[str] = None
    lppwaehr: Optional[str] = None
    aendstatus: Optional[str] = None
    abteilung: Optional[int] = None
    lkw: Optional[str] = None
    anh: Optional[str] = None
    bereich: Optional[int] = None
    plombe1: Optional[str] = None
    plombe2: Optional[str] = None
    kmleer: Optional[float] = None
    kmleermaut: Optional[float] = None
    kmlastist: Optional[float] = None
    kmleerist: Optional[float] = None

    @field_validator("tlade_datum", "entbisdat", mode="before")
    @classmethod
    def parse_date(cls, value):
        if isinstance(value, str):
            return datetime.strptime(value, "%Y%m%d")
        return value

    @field_validator("tourzeit", "entbiszeit", mode="before")
    @classmethod
    def parse_time(cls, value):
        if isinstance(value, str):
            return datetime.strptime(value, "%H%M").time().strftime("%H%M")
        return value

    @field_validator("aendstatus")
    @classmethod
    def validate_aendstatus(cls, value):
        if value and value not in {"N", "A"}:
            raise ValueError("Invalid Aendstatus value")
        return value

    @field_validator("exportiert")
    @classmethod
    def validate_exportiert(cls, value):
        if value in {True, "J", "j"}:
            return "J"
        elif value in {False, "N", "n"}:
            return "N"
        elif value is None:
            return None
        raise ValueError("exportiert must be 'J', 'N', True, False or None")



import os
import shutil
import tempfile

from typing import Union, Tuple, List
from pydantic.dataclasses import dataclass
from tms_integration.utils.sftp import SftpBase

from .models import LisIn


@dataclass
class LisWinSped(SftpBase):
    import_dest_folder: str
    output_target_folder: str

    def import_auftrag(self, payload: LisIn, import_prefix: str = None):
        with tempfile.NamedTemporaryFile(
            mode="w",
            encoding="cp1252",  # FORCE ANSI encoding
            prefix=import_prefix,
            suffix=".txt",
            delete=False,
        ) as tmp_file:
            tmp_file.write(payload.generate_txt())
            tmp_file.close()
            self.import_file(tmp_file.name, self.import_dest_folder)

    def import_document(self, dms_payload, file: str, import_prefix: str = None):
        with tempfile.NamedTemporaryFile(
            mode="w",
            encoding="cp1252",  # FORCE ANSI encoding
            prefix=import_prefix,
            suffix=".txt",
            delete=False,
        ) as tmp_file:
            tmp_file.write(dms_payload.generate_txt())
            tmp_file.close()
            self.import_file(tmp_file.name, self.import_dest_folder)

        # send the file as well
        self.import_file(file, self.import_dest_folder)

    def export_auftrag(
        self, identifier: str
    ) -> Union[Tuple[None, None], Tuple[str, str]]:
        output_files = self.get_all_files(self.output_target_folder)
        dest_path = os.path.join(os.getcwd(), "tmp", "output")
        if os.path.exists(dest_path):
            shutil.rmtree(dest_path)
        os.makedirs(dest_path)
        for file in output_files:
            filename = os.path.basename(file)
            self.export_file(file, os.path.join(dest_path, filename))
            with open(os.path.join(dest_path, filename), "r", encoding="cp1252") as txt:
                text = txt.read()
                if identifier in text:
                    return (file, text)

        return None, None

    def export_auftrag_tour(self, identifier: str) -> List[Tuple[str, str]]:
        output: List[Tuple[str, str]] = []
        output_files = self.get_all_files(self.output_target_folder)
        dest_path = os.path.join(os.getcwd(), "tmp", "output")
        if os.path.exists(dest_path):
            shutil.rmtree(dest_path)
        os.makedirs(dest_path)
        for file in output_files:
            filename = os.path.basename(file)
            self.export_file(file, os.path.join(dest_path, filename))
            with open(os.path.join(dest_path, filename), "r", encoding="cp1252") as txt:
                text = txt.read()
                if identifier in text:
                    output.append((file, text))

        return output



from pydantic import BaseModel, field_validator
from typing import Optional, Literal
from datetime import datetime, time


class Auftrag(BaseModel):
    satzart: Literal["AUFTR"] = "AUFTR"
    referenz: str
    tladenr: str
    aufnr: Optional[int] = None
    liefnr: Optional[str] = None
    lieftxt: Optional[str] = None
    aufdatum: Optional[datetime] = None
    belvondat: Optional[datetime] = None
    belvonzeit: Optional[time] = None
    belbisdat: Optional[datetime] = None
    belbiszeit: Optional[time] = None
    entvondat: Optional[datetime] = None
    entvonzeit: Optional[time] = None
    entbisdat: Optional[datetime] = None
    entbiszeit: Optional[time] = None
    lagnr: Optional[str] = None
    lagauftrag: Optional[str] = None
    verkart: Optional[str] = None
    frankatur: Optional[str] = None
    fnach: Optional[float] = None
    fnachfrei: Optional[float] = None
    wnachfrei: Optional[float] = None
    bar: Optional[bool] = None
    vorkosten: Optional[float] = None
    gefahrgut: Optional[bool] = None
    kuehlgut: Optional[bool] = None
    direkt: Optional[bool] = None
    komnr: Optional[str] = None
    chargenr: Optional[str] = None
    km: Optional[float] = None
    aufklasse: Optional[str] = None
    kostenst: Optional[int] = None
    refnr: Optional[str] = None
    rel: Optional[str] = None
    aendstatus: Optional[str] = None
    wwwaehr: Optional[str] = None
    wauftragid: Optional[int] = None
    ktraeger: Optional[int] = None
    fobfrank: Optional[str] = None
    dispoinfo1: Optional[str] = None
    dispoinfo2: Optional[str] = None
    dispoinfo3: Optional[str] = None
    dispoinfo4: Optional[str] = None
    ffpausch: Optional[float] = None
    fzpausch: Optional[float] = None
    belfixdat: Optional[datetime] = None
    belfixzeit: Optional[time] = None
    entfixdat: Optional[datetime] = None
    entfixzeit: Optional[time] = None
    tmittel: Optional[int] = None
    tvpausch: Optional[float] = None
    aufunternr: Optional[int] = None
    ablauf: Optional[int] = None
    aufart: Optional[str] = None
    borderonr: Optional[str] = None
    belsolldat: Optional[datetime] = None
    belsollzeit: Optional[time] = None
    entsolldat: Optional[datetime] = None
    entsollzeit: Optional[time] = None
    dsinr: Optional[str] = None
    erfdr: Optional[int] = None
    abteilung: Optional[int] = None
    bereich: Optional[int] = None
    ffpwaehr: Optional[str] = None
    fzpwaehr: Optional[str] = None
    dosnr: Optional[str] = None
    dispoinfo5: Optional[str] = None
    dispoinfo6: Optional[str] = None
    dispoinfo7: Optional[str] = None
    dispoinfo8: Optional[str] = None
    dispoinfo9: Optional[str] = None
    dispoinfo10: Optional[str] = None
    tourlfdnr: Optional[int] = None
    belsolldatbis: Optional[datetime] = None
    belsollzeitbis: Optional[time] = None
    entsolldatbis: Optional[datetime] = None
    entsollzeitbis: Optional[time] = None
    belfixdatbis: Optional[datetime] = None
    belfixzeitbis: Optional[time] = None
    entfixdatbis: Optional[datetime] = None
    entfixzeitbis: Optional[time] = None
    fnachw: Optional[str] = None
    wnachfreiw: Optional[str] = None
    vdisintnr: Optional[int] = None
    renettoant: Optional[float] = None
    gsnettoant: Optional[float] = None
    eigenuser: Optional[str] = None
    sollgew: Optional[float] = None
    austourent: Optional[int] = None
    vdisnrext: Optional[str] = None
    umweltgef: Optional[bool] = None
    kontotab: Optional[int] = None
    dlaart: Optional[str] = None

    @field_validator(
        "aufdatum",
        "belvondat",
        "belbisdat",
        "entvondat",
        "entbisdat",
        "belfixdat",
        "entfixdat",
        "belsolldat",
        "entsolldat",
        "belsolldatbis",
        "entsolldatbis",
        "belfixdatbis",
        "entfixdatbis",
        mode="before",
    )
    @classmethod
    def parse_date(cls, value):
        if isinstance(value, str):
            return datetime.strptime(value, "%Y%m%d")
        return value

    @field_validator(
        "belvonzeit",
        "belbiszeit",
        "entvonzeit",
        "entbiszeit",
        "belfixzeit",
        "entfixzeit",
        "belsollzeit",
        "entsollzeit",
        "belsollzeitbis",
        "entsollzeitbis",
        "belfixzeitbis",
        "entfixzeitbis",
        mode="before",
    )
    @classmethod
    def parse_time(cls, value):
        if isinstance(value, str):
            return datetime.strptime(value, "%H%M").time()
        return value

    @field_validator("aendstatus")
    @classmethod
    def validate_aendstatus(cls, value):
        if value and value not in {"N", "A", "L", "W"}:
            raise ValueError("Invalid Aendstatus value")
        return value

    @field_validator("bar", "gefahrgut", "kuehlgut", "direkt", "umweltgef")
    @classmethod
    def validate_boolean(cls, value):
        if value not in {None, True, False}:
            raise ValueError("Invalid boolean value")
        if value == False:
            return "N"  # Nein
        elif value == True:
            return "J"  # Ja
        return value



from typing import List, Optional
from datetime import datetime, time
from pydantic import BaseModel

from .types import *


class LisIn(BaseModel):
    start: Start
    records: List[RecordTypes] = []
    ende: Optional[Ende] = None

    def validate_records(self):
        raise NotImplemented

    @staticmethod
    def model_to_line(model):
        fields = []
        for field_key in model.__fields__.keys():
            field_value = model.dict().get(field_key)
            if field_value is not None:
                if isinstance(field_value, datetime):
                    fields.append(field_value.strftime("%Y%m%d"))
                elif isinstance(field_value, time):
                    fields.append(field_value.strftime("%H%M"))
                else:
                    fields.append(str(field_value))
            else:
                fields.append("")
        return "|".join(fields)

    def generate_txt(self) -> str:
        self.validate_records()
        lines = []

        lines.append(self.model_to_line(self.start))
        for record in self.records:
            lines.append(self.model_to_line(record))
        if self.ende:
            lines.append(self.model_to_line(self.ende))

        return "\n".join(lines)


class LisInAuftrag(LisIn):
    mandatory_records_type: List = [Auftrag, Adr, Ladeli, Posit]

    def validate_records(self):
        records_type = {type(r) for r in self.records}
        types_missing = set(self.mandatory_records_type) - records_type
        if types_missing:
            raise ValueError(f"Some mandatory types are missing: {types_missing}")


class LisInDMS(LisIn):
    mandatory_records_type: List = [DmsDok, DmsRef, DmsSw]

    def validate_records(self):
        records_type = {type(r) for r in self.records}
        types_missing = set(self.mandatory_records_type) - records_type
        if types_missing:
            raise ValueError(f"Some mandatory types are missing: {types_missing}")



from setuptools import setup, find_packages
import os
import sys


def get_requirements(path):
    with open(path) as f:
        return f.read().splitlines()


base_requirements = get_requirements(
    os.path.join("tms_integration", "requirements.txt")
)

# Define the requirements for each optional component
lis_winsped_requirements = get_requirements(
    os.path.join("tms_integration", "lis_winsped", "requirements.txt")
)
# Assuming carlo has its own requirements.txt
carlo_requirements = get_requirements(
    os.path.join("tms_integration", "carlo", "requirements.txt")
)

setup(
    name="tms_integration",
    version="0.1.3",
    description="A library for TMS integration",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author="F-ONE Group",
    author_email="admin@f-one.group",
    url="https://github.com/F-ONE-Group/tms_integration/tree/pypi",
    packages=find_packages(exclude=["tests*"]),
    install_requires=base_requirements,
    extras_require={
        "lis_winsped": lis_winsped_requirements,
        "carlo": carlo_requirements,
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)
